@@ -6,7 +6,7 @@ Utilities for working with terminals, strings, and Enums.
 
 from __future__ import annotations
 
-from enum import Flag, Enum, EnumMeta
+from enum import Flag, EnumMeta
 from shutil import get_terminal_size
 from time import monotonic
 from typing import Any, Callable, TypeVar, List
@@ -144,11 +144,6 @@ class Terminal:  # pylint: disable=R0903
         return self._width
 
 
-class ValueSource(Enum):
-    CLI = 'cli'
-    ENV = 'env'
-
-
 def str_to_bool(value: str) -> bool:
     try:
         return bool(int(value))
